@@ -129,7 +129,7 @@ load_WSM <- function(file, quality = c("A", "B", "C", "D", "E", "X"),
   out <- utils::read.csv(file) |>
     dplyr::rename_with(tolower) |>
     dplyr::filter(
-      !is.na(lat), !is.na(lon),
+      # !is.na(lat), !is.na(lon),
       dplyr::between(lat, lat_range[1], lat_range[2]),
       dplyr::between(lon, lon_range[1], lon_range[2]),
       dplyr::between(depth, depth_range[1], depth_range[2]),
@@ -146,23 +146,20 @@ load_WSM <- function(file, quality = c("A", "B", "C", "D", "E", "X"),
         regime == "TF" ~ "T",
         regime == "TS" ~ "TS",
         regime == "SS" ~ "S",
-        regime == "U" ~ NA,
-        TRUE ~ NA_character_,
-        .default = regime
+        regime == "U" ~ NA_character_,
+        TRUE ~ regime,
       ),
       unc = ifelse(is.na(sd), quality.numeric, sd),
       unc = ifelse(unc > quality.numeric, quality.numeric, unc),
       unc = ifelse(unc == 0, 15, unc),
       unc = as.numeric(unc),
-      dplyr::across(dplyr::where(is.character), function(x) {
-        ifelse(x == "", NA, x)
-      }),
-      eq_mag = as.numeric(eq_mag),
+      dplyr::across(dplyr::where(is.character), ~ dplyr::na_if(.x, "")),
+      eq_mag = suppressWarnings(as.numeric(eq_mag)),
       # date = ymd(date),
       # time = hms(time)
     ) |>
     dplyr::filter(quality %in% qlt_flt, regime %in% reg_flt) |>
-    sf::st_as_sf(coords = c("lon", "lat"), crs = sf::st_crs("WGS84"), remove = FALSE)
+    sf::st_as_sf(coords = c("lon", "lat"), crs = sf::st_crs("WGS84"), remove = FALSE, na.fail = FALSE)
 
   if ("isc_id" %in% colnames(out)) {
     dplyr::select(out, id, lat, lon, azi, unc, type, depth, quality, regime, isc_id, locality:dist)
@@ -243,18 +240,16 @@ load_WSM2016 <- function(file, quality = c("A", "B", "C", "D", "E"), lat_range =
         regime == "TS" ~ "TS",
         regime == "SS" ~ "S",
         regime == "U" ~ NA,
-        TRUE ~ NA_character_,
-        .default = regime
+        regime == "U" ~ NA_character_,
+        TRUE ~ regime
       ),
       quality.numeric = parse_wsm_quality(quality),
       unc = ifelse(is.na(sd), quality.numeric, sd),
       unc = ifelse(unc > quality.numeric, quality.numeric, unc),
       unc = ifelse(unc == 0, 15, unc),
-      unc = as.numeric(unc),
-      dplyr::across(dplyr::where(is.character), function(x) {
-        ifelse(x == "", NA, x)
-      }),
-      eq_mag = as.numeric(eq_mag),
+      unc = suppressWarnings(as.numeric(unc)),
+      dplyr::across(dplyr::where(is.character), ~ dplyr::na_if(.x, "")),
+      eq_mag = suppressWarnings(as.numeric(eq_mag)),
       # date = ymd(date),
       # time = hms(time)
     ) |>
@@ -427,10 +422,11 @@ stress_analysis <- function(x, PoR, type = c("none", "in", "out", "right", "left
 #' # one line:
 #' subset(plates, pair == "af-eu") |>
 #'   smoothr::densify() |>
-#'   line_azimuth()
+#'   line_azimuth() |>
+#'   head()
 #'
 #' # multiple lines:
-#' lines_azimuths(plates[1:5, ])
+#' lines_azimuths(plates[1:5, ]) |> head()
 NULL
 
 #' @rdname line_azimuth
@@ -443,30 +439,73 @@ line_azimuth <- function(x, warn = TRUE) {
     sf::st_coordinates()
 
   n <- nrow(mat)
-  # a <- numeric(n - 1)
-  # for (i in 1:(n - 1)) {
-  #   a[i] <- get_azimuth(mat[i, 2], mat[i, 1], mat[i + 1, 2], mat[i + 1, 1])
-  # }
-  a <- vapply(
-    1:(n - 1), function(i) {
-      get_azimuth(mat[i, 2], mat[i, 1], mat[i + 1, 2], mat[i + 1, 1])
-    },
-    numeric(1)
+  a <- get_azimuth(
+    mat[1:(n - 1), 2], mat[1:(n - 1), 1],
+    mat[2:n, 2], mat[2:n, 1]
   )
 
-  data.frame(
-    x = mat[1:(n - 1), 1],
-    y = mat[1:(n - 1), 2],
-    azi = a
-  ) |>
-    cbind(sf::st_drop_geometry(x[1, ])) |>
-    sf::st_as_sf(coords = c("x", "y"), crs = sf::st_crs(x))
+  ldf <- cbind(
+    data.frame(
+      x = mat[1:(n - 1), 1],
+      y = mat[1:(n - 1), 2],
+      azi = a
+    ),
+    sf::st_drop_geometry(x[1, ])
+  )
+
+  sf::st_as_sf(ldf, coords = c("x", "y"), crs = sf::st_crs(x))
 }
 
 #' @rdname line_azimuth
 #' @export
 lines_azimuths <- function(x) {
-  la <- split(x, 1:nrow(x)) |>
-    lapply(line_azimuth, warn = FALSE)
-  do.call(rbind, la)
+  la <- lapply(split(x, 1:nrow(x)), line_azimuth, warn = FALSE)
+  dplyr::bind_rows(la)
+}
+
+
+#' Weighting Factors
+#'
+#' Helper function to transform uncertainty angles into weighting factors
+#'
+#' @param x numeric. Uncertainty angle in degrees.
+#' @param method character. One of `"linear-inverse"` (the default), `"inverse"`, `"cosine"`, or `"none"` (no transformation).
+#' @param max.err numeric. The maximum expected error for x (90 by default).
+#'
+#' @details
+#' Linear inverse: \eqn{w = 1 - x/\sigma}, where \eqn{\sigma} is the maximum error expected for \eqn{x} (e.g. \eqn{90^\circ}).
+#'
+#' Inverse: \eqn{w = 1/x}
+#'
+#' Cosine: \eqn{w = \cos{x}}
+#'
+#'
+#' @returns numeric
+#' @export
+#'
+#' @examples
+#' x <- seq(0, 90, 1)
+#'
+#' plot(x, weighting(x, "inverse"),
+#'   col = 1, type = "l",
+#'   xlab = "Uncertainty angle in degrees", ylab = "weight"
+#' )
+#' lines(x, weighting(x, "cosine"), col = 2)
+#' lines(x, weighting(x, "linear-inverse"), col = 3)
+#' legend("topright",
+#'   col = 1:3, lty = 1,
+#'   legend = c("inverse", "cosine", "linear-inverse")
+#' )
+weighting <- function(x, method = c("linear-inverse", "inverse", "cosine", "none"), max.err = 90) {
+  method <- match.arg(method)
+
+  if (method == "linear-inverse") {
+    1 - abs(x) / max.err
+  } else if (method == "inverse") {
+    1 / abs(x)
+  } else if (method == "cosine") {
+    cos(abs(x) * pi / 180)
+  } else {
+    x
+  }
 }
